@@ -15,7 +15,7 @@ def test_creating_view(qtbot):
     Test creating dims view.
     """
     ndim = 4
-    dims = Dims(ndim)
+    dims = Dims(ndim=ndim)
     view = QtDims(dims)
 
     qtbot.addWidget(view)
@@ -40,7 +40,7 @@ def test_changing_ndim(qtbot):
     Test changing the number of dimensions
     """
     ndim = 4
-    view = QtDims(Dims(ndim))
+    view = QtDims(Dims(ndim=ndim))
 
     qtbot.addWidget(view)
 
@@ -69,25 +69,26 @@ def test_changing_ndim(qtbot):
 
 def test_changing_focus(qtbot):
     """Test changing focus updates the dims.last_used prop."""
-    # too-few dims, should have no sliders to update
+    # Initialize to 0th axis
     ndim = 2
-    view = QtDims(Dims(ndim))
+    view = QtDims(Dims(ndim=ndim))
     qtbot.addWidget(view)
-    assert view.last_used is None
-    view.focus_down()
-    view.focus_up()
-    assert view.last_used is None
+    assert view.dims.last_used == 0
+    view.dims._focus_down()
+    view.dims._focus_up()
+    assert view.dims.last_used == 0
 
     view.dims.ndim = 5
-    assert view.last_used == 2
-    view.focus_down()
-    assert view.last_used == 1
-    view.focus_up()
-    assert view.last_used == 2
-    view.focus_up()
-    assert view.last_used == 0
-    view.focus_down()
-    assert view.last_used == 2
+    view.dims.last_used = 2
+    assert view.dims.last_used == 2
+    view.dims._focus_down()
+    assert view.dims.last_used == 1
+    view.dims._focus_up()
+    assert view.dims.last_used == 2
+    view.dims._focus_up()
+    assert view.dims.last_used == 0
+    view.dims._focus_down()
+    assert view.dims.last_used == 2
 
 
 def test_changing_display(qtbot):
@@ -95,7 +96,7 @@ def test_changing_display(qtbot):
     Test changing the displayed property of an axis
     """
     ndim = 4
-    view = QtDims(Dims(ndim))
+    view = QtDims(Dims(ndim=ndim))
     qtbot.addWidget(view)
 
     assert view.nsliders == view.dims.ndim
@@ -124,7 +125,7 @@ def test_slider_values(qtbot):
     Test the values of a slider stays matched to the values of the dims point.
     """
     ndim = 4
-    view = QtDims(Dims(ndim))
+    view = QtDims(Dims(ndim=ndim))
     qtbot.addWidget(view)
 
     # Check that values of the dimension slider matches the values of the
@@ -148,7 +149,7 @@ def test_slider_range(qtbot):
     Tests range of the slider is matched to the range of the dims
     """
     ndim = 4
-    view = QtDims(Dims(ndim))
+    view = QtDims(Dims(ndim=ndim))
     qtbot.addWidget(view)
 
     # Check the maximum allowed value of the slider is one less
@@ -171,8 +172,8 @@ def test_singleton_dims(qtbot):
     Test singleton dims causes no slider.
     """
     ndim = 4
-    dims = Dims(ndim)
-    dims.set_range(0, (0, 1, 1))
+    dims = Dims(ndim=ndim)
+    dims.set_range(0, (0, 0, 1))
     view = QtDims(dims)
     qtbot.addWidget(view)
 
@@ -207,7 +208,7 @@ def test_order_when_changing_ndim(qtbot):
     Test order of the sliders when changing the number of dimensions.
     """
     ndim = 4
-    view = QtDims(Dims(ndim))
+    view = QtDims(Dims(ndim=ndim))
     qtbot.addWidget(view)
 
     # Check that values of the dimension slider matches the values of the
@@ -246,7 +247,7 @@ def test_update_dims_labels(qtbot):
     and vice versa.
     """
     ndim = 4
-    view = QtDims(Dims(ndim))
+    view = QtDims(Dims(ndim=ndim))
     qtbot.addWidget(view)
     view.dims.axis_labels = list('TZYX')
     assert [w.axis_label.text() for w in view.slider_widgets] == list('TZYX')
@@ -261,12 +262,12 @@ def test_update_dims_labels(qtbot):
 def test_slider_press_updates_last_used(qtbot):
     """pressing on the slider should update the dims.last_used property"""
     ndim = 5
-    view = QtDims(Dims(ndim))
+    view = QtDims(Dims(ndim=ndim))
     qtbot.addWidget(view)
 
     for i, widg in enumerate(view.slider_widgets):
         widg.slider.sliderPressed.emit()
-        assert view.last_used == i
+        assert view.dims.last_used == i
 
 
 @pytest.mark.skipif(
@@ -276,17 +277,16 @@ def test_slider_press_updates_last_used(qtbot):
 def test_play_button(qtbot):
     """test that the play button and its popup dialog work"""
     ndim = 3
-    view = QtDims(Dims(ndim))
+    view = QtDims(Dims(ndim=ndim))
     qtbot.addWidget(view)
     button = view.slider_widgets[0].play_button
     qtbot.mouseClick(button, Qt.LeftButton)
     qtbot.waitSignal(view._animation_thread.started, timeout=5000)
-    qtbot.wait(200)
-    assert view.is_playing
+
     with qtbot.waitSignal(view._animation_thread.finished, timeout=7000):
         qtbot.mouseClick(button, Qt.LeftButton)
 
-    qtbot.wait(200)
+    qtbot.wait(100)
     assert not view.is_playing
 
     with patch.object(button.popup, 'show_above_mouse') as mock_popup:
@@ -294,22 +294,22 @@ def test_play_button(qtbot):
         mock_popup.assert_called_once()
 
 
-def test_slice_labels(make_test_viewer):
-    viewer = make_test_viewer()
-    np.random.seed(0)
-    data = np.random.random((20, 10, 10))
-    viewer.add_image(data)
-    view = viewer.window.qt_viewer
+def test_slice_labels(qtbot):
+    ndim = 4
+    dims = Dims(ndim=ndim)
+    dims.set_range(0, (0, 19, 1))
+    view = QtDims(dims)
+    qtbot.addWidget(view)
 
     # make sure the totslice_label is showing the correct number
-    assert int(view.dims.slider_widgets[0].totslice_label.text()) == 19
+    assert int(view.slider_widgets[0].totslice_label.text()) == 19
 
     # make sure setting the dims.point updates the slice label
-    label_edit = view.dims.slider_widgets[0].curslice_label
-    viewer.dims.set_point(0, 15)
+    label_edit = view.slider_widgets[0].curslice_label
+    dims.set_point(0, 15)
     assert int(label_edit.text()) == 15
 
     # make sure setting the current slice label updates the model
     label_edit.setText(str(8))
     label_edit.editingFinished.emit()
-    assert viewer.dims.point[0] == 8
+    assert dims.point[0] == 8
