@@ -15,7 +15,9 @@ class ImageLayerNode:
     def __init__(self, custom_node: Node = None):
         self._custom_node = custom_node
         self._image_node = ImageNode(None, method='auto')
-        self._volume_node = VolumeNode(np.zeros((1, 1, 1)), clim=[0, 1])
+        self._volume_node = VolumeNode(
+            np.zeros((1, 1, 1), dtype=np.float32), clim=[0, 1]
+        )
 
     def get_node(self, ndisplay: int) -> Node:
 
@@ -49,6 +51,18 @@ class VispyImageLayer(VispyBaseLayer):
         self.layer.events.gamma.connect(self._on_gamma_change)
         self.layer.events.iso_threshold.connect(self._on_iso_threshold_change)
         self.layer.events.attenuation.connect(self._on_attenuation_change)
+        self.layer.experimental_slicing_plane.events.enabled.connect(
+            self._on_experimental_slicing_plane_enabled_change
+        )
+        self.layer.experimental_slicing_plane.events.position.connect(
+            self._on_experimental_slicing_plane_position_change
+        )
+        self.layer.experimental_slicing_plane.events.thickness.connect(
+            self._on_experimental_slicing_plane_thickness_change
+        )
+        self.layer.experimental_slicing_plane.events.normal.connect(
+            self._on_experimental_slicing_plane_normal_change
+        )
 
         self._on_display_change()
         self._on_data_change()
@@ -143,6 +157,32 @@ class VispyImageLayer(VispyBaseLayer):
         if isinstance(self.node, VolumeNode):
             self.node.attenuation = self.layer.attenuation
 
+    def _on_experimental_slicing_plane_enabled_change(self, event=None):
+        if isinstance(self.node, VolumeNode):
+            if self.layer.experimental_slicing_plane.enabled is True:
+                raycasting_mode = 'plane'
+            else:
+                raycasting_mode = 'volume'
+            self.node.raycasting_mode = raycasting_mode
+
+    def _on_experimental_slicing_plane_thickness_change(self, event=None):
+        if isinstance(self.node, VolumeNode):
+            self.node.plane_thickness = (
+                self.layer.experimental_slicing_plane.thickness
+            )
+
+    def _on_experimental_slicing_plane_position_change(self, event=None):
+        if isinstance(self.node, VolumeNode):
+            self.node.plane_position = (
+                self.layer.experimental_slicing_plane.position
+            )
+
+    def _on_experimental_slicing_plane_normal_change(self, event=None):
+        if isinstance(self.node, VolumeNode):
+            self.node.plane_normal = (
+                self.layer.experimental_slicing_plane.normal
+            )
+
     def reset(self, event=None):
         self._reset_base()
         self._on_interpolation_change()
@@ -150,6 +190,10 @@ class VispyImageLayer(VispyBaseLayer):
         self._on_contrast_limits_change()
         self._on_gamma_change()
         self._on_rendering_change()
+        self._on_experimental_slicing_plane_enabled_change()
+        self._on_experimental_slicing_plane_position_change()
+        self._on_experimental_slicing_plane_normal_change()
+        self._on_experimental_slicing_plane_thickness_change()
 
     def downsample_texture(self, data, MAX_TEXTURE_SIZE):
         """Downsample data based on maximum allowed texture size.
