@@ -1,5 +1,7 @@
 import itertools
+from dataclasses import dataclass
 from tempfile import TemporaryDirectory
+from typing import List
 
 import numpy as np
 import pandas as pd
@@ -13,6 +15,7 @@ from skimage import data
 
 from napari._tests.utils import check_layer_world_data_extent
 from napari.layers import Labels
+from napari.layers.image._image_constants import Rendering
 from napari.utils import Colormap
 from napari.utils.colormaps import low_discrepancy_image
 
@@ -55,7 +58,7 @@ def test_3D_labels():
 
     layer._slice_dims(ndisplay=3)
     assert layer._ndisplay == 3
-    assert layer.editable is False
+    assert layer.editable is True
     assert layer.mode == 'pan_zoom'
 
 
@@ -268,7 +271,9 @@ def test_properties():
     assert isinstance(layer.properties, dict)
     assert len(layer.properties) == 0
 
-    properties = {'class': ['Background'] + [f'Class {i}' for i in range(20)]}
+    properties = {
+        'class': np.array(['Background'] + [f'Class {i}' for i in range(20)])
+    }
     label_index = {i: i for i in range(len(properties['class']))}
     layer = Labels(data, properties=properties)
     assert isinstance(layer.properties, dict)
@@ -332,7 +337,9 @@ def test_multiscale_properties():
     assert isinstance(layer.properties, dict)
     assert len(layer.properties) == 0
 
-    properties = {'class': ['Background'] + [f'Class {i}' for i in range(20)]}
+    properties = {
+        'class': np.array(['Background'] + [f'Class {i}' for i in range(20)])
+    }
     label_index = {i: i for i in range(len(properties['class']))}
     layer = Labels(data, properties=properties)
     assert isinstance(layer.properties, dict)
@@ -614,31 +621,32 @@ def test_show_selected_label():
 
 
 def test_paint():
-    """Test painting labels with different square brush sizes."""
+    """Test painting labels with different circle brush sizes."""
     np.random.seed(0)
     data = np.random.randint(20, size=(10, 15))
     data[:10, :10] = 1
     layer = Labels(data)
 
-    with pytest.warns(FutureWarning):
-        layer.brush_shape = 'square'
     assert np.unique(layer.data[:5, :5]) == 1
     assert np.unique(layer.data[5:10, 5:10]) == 1
 
     layer.brush_size = 9
     layer.paint([0, 0], 2)
-    assert np.unique(layer.data[:5, :5]) == 2
+    assert np.unique(layer.data[:4, :4]) == 2
     assert np.unique(layer.data[5:10, 5:10]) == 1
 
     layer.brush_size = 10
     layer.paint([0, 0], 2)
-    assert np.unique(layer.data[:6, :6]) == 2
+    assert np.unique(layer.data[0:6, 0:3]) == 2
+    assert np.unique(layer.data[0:3, 0:6]) == 2
     assert np.unique(layer.data[6:10, 6:10]) == 1
 
     layer.brush_size = 19
     layer.paint([0, 0], 2)
-    assert np.unique(layer.data[:5, :5]) == 2
-    assert np.unique(layer.data[5:10, 5:10]) == 2
+    assert np.unique(layer.data[0:4, 0:10]) == 2
+    assert np.unique(layer.data[0:10, 0:4]) == 2
+    assert np.unique(layer.data[3:7, 3:7]) == 2
+    assert np.unique(layer.data[7:10, 7:10]) == 1
 
 
 def test_paint_with_preserve_labels():
@@ -646,30 +654,23 @@ def test_paint_with_preserve_labels():
     data = np.zeros((15, 10), dtype=np.uint32)
     data[:3, :3] = 1
     layer = Labels(data)
-    with pytest.warns(FutureWarning):
-        layer.brush_shape = 'square'
+
     layer.preserve_labels = True
     assert np.unique(layer.data[:3, :3]) == 1
 
     layer.brush_size = 9
     layer.paint([0, 0], 2)
 
-    assert np.unique(layer.data[3:5, 0:5]) == 2
-    assert np.unique(layer.data[0:5, 3:5]) == 2
+    assert np.unique(layer.data[3:5, 0:3]) == 2
+    assert np.unique(layer.data[0:3, 3:5]) == 2
     assert np.unique(layer.data[:3, :3]) == 1
 
 
-@pytest.mark.parametrize(
-    "brush_shape, expected_sum",
-    [("circle", [41, 137, 137, 41, 349]), ("square", [36, 144, 169, 36, 400])],
-)
-def test_paint_2d(brush_shape, expected_sum):
-    """Test painting labels with circle/square brush."""
+def test_paint_2d():
+    """Test painting labels with circle brush."""
     data = np.zeros((40, 40), dtype=np.uint32)
     layer = Labels(data)
     layer.brush_size = 12
-    with pytest.warns(FutureWarning):
-        layer.brush_shape = brush_shape
     layer.mode = 'paint'
     layer.paint((0, 0), 3)
 
@@ -685,43 +686,31 @@ def test_paint_2d(brush_shape, expected_sum):
     layer.brush_size = 20
     layer.paint((15, 27), 7)
 
-    assert np.sum(layer.data[:8, :8] == 3) == expected_sum[0]
-    assert np.sum(layer.data[9:22, 2:15] == 4) == expected_sum[1]
-    assert np.sum(layer.data[24:37, 2:15] == 5) == expected_sum[2]
-    assert np.sum(layer.data[33:, 33:] == 6) == expected_sum[3]
-    assert np.sum(layer.data[5:26, 17:38] == 7) == expected_sum[4]
+    assert np.sum(layer.data[:8, :8] == 3) == 41
+    assert np.sum(layer.data[9:22, 2:15] == 4) == 137
+    assert np.sum(layer.data[24:37, 2:15] == 5) == 137
+    assert np.sum(layer.data[33:, 33:] == 6) == 41
+    assert np.sum(layer.data[5:26, 17:38] == 7) == 349
 
 
 @pytest.mark.timeout(1)
-@pytest.mark.parametrize(
-    "brush_shape, expected_sum",
-    [("circle", 411), ("square", 432)],
-)
-def test_paint_2d_xarray(brush_shape, expected_sum):
+def test_paint_2d_xarray():
     """Test the memory usage of painting an xarray indirectly via timeout."""
     data = xr.DataArray(np.zeros((3, 3, 1024, 1024), dtype=np.uint32))
 
     layer = Labels(data)
     layer.brush_size = 12
-    with pytest.warns(FutureWarning):
-        layer.brush_shape = brush_shape
     layer.mode = 'paint'
     layer.paint((1, 1, 512, 512), 3)
     assert isinstance(layer.data, xr.DataArray)
-    assert layer.data.sum() == expected_sum
+    assert layer.data.sum() == 411
 
 
-@pytest.mark.parametrize(
-    "brush_shape, expected_sum",
-    [("circle", [137, 1189, 1103]), ("square", [144, 1728, 1548])],
-)
-def test_paint_3d(brush_shape, expected_sum):
-    """Test painting labels with circle/square brush on 3D image."""
+def test_paint_3d():
+    """Test painting labels with circle brush on 3D image."""
     data = np.zeros((30, 40, 40), dtype=np.uint32)
     layer = Labels(data)
     layer.brush_size = 12
-    with pytest.warns(FutureWarning):
-        layer.brush_shape = brush_shape
     layer.mode = 'paint'
 
     # Paint in 2D
@@ -736,9 +725,9 @@ def test_paint_3d(brush_shape, expected_sum):
     layer.preserve_labels = True
     layer.paint((10, 15, 15), 5)
 
-    assert np.sum(layer.data[4:17, 4:17, 4:17] == 3) == expected_sum[0]
-    assert np.sum(layer.data[4:17, 19:32, 4:17] == 4) == expected_sum[1]
-    assert np.sum(layer.data[4:17, 9:32, 9:32] == 5) == expected_sum[2]
+    assert np.sum(layer.data[4:17, 4:17, 4:17] == 3) == 137
+    assert np.sum(layer.data[4:17, 19:32, 4:17] == 4) == 1189
+    assert np.sum(layer.data[4:17, 9:32, 9:32] == 5) == 1103
 
 
 def test_fill():
@@ -765,6 +754,29 @@ def test_value():
     assert value == data[0, 0]
 
 
+@pytest.mark.parametrize(
+    'position,view_direction,dims_displayed,world',
+    [
+        ([10, 5, 5], [1, 0, 0], [0, 1, 2], False),
+        ([10, 5, 5], [1, 0, 0], [0, 1, 2], True),
+        ([0, 10, 5, 5], [0, 1, 0, 0], [1, 2, 3], True),
+    ],
+)
+def test_value_3d(position, view_direction, dims_displayed, world):
+    """get_value should return label value in 3D"""
+    data = np.zeros((20, 20, 20), dtype=int)
+    data[0:10, 0:10, 0:10] = 1
+    layer = Labels(data)
+    layer._slice_dims([0, 0, 0], ndisplay=3)
+    value = layer.get_value(
+        position,
+        view_direction=view_direction,
+        dims_displayed=dims_displayed,
+        world=world,
+    )
+    assert value == 1
+
+
 def test_message():
     """Test converting value and coords to message."""
     np.random.seed(0)
@@ -794,10 +806,9 @@ def test_world_data_extent():
 
 
 @pytest.mark.parametrize(
-    'brush_shape, brush_size, mode, selected_label, preserve_labels, n_dimensional',
+    'brush_size, mode, selected_label, preserve_labels, n_dimensional',
     list(
         itertools.product(
-            ['square', 'circle'],
             list(range(1, 22, 5)),
             ['fill', 'erase', 'paint'],
             [1, 20, 100],
@@ -807,7 +818,6 @@ def test_world_data_extent():
     ),
 )
 def test_undo_redo(
-    brush_shape,
     brush_size,
     mode,
     selected_label,
@@ -817,8 +827,6 @@ def test_undo_redo(
     blobs = data.binary_blobs(length=64, volume_fraction=0.3, n_dim=3)
     layer = Labels(blobs)
     data_history = [blobs.copy()]
-    with pytest.warns(FutureWarning):
-        layer.brush_shape = brush_shape
     layer.brush_size = brush_size
     layer.mode = mode
     layer.selected_label = selected_label
@@ -865,8 +873,6 @@ def test_ndim_paint():
     test_array = np.zeros((5, 6, 7, 8), dtype=int)
     layer = Labels(test_array)
     layer.n_edit_dimensions = 3
-    with pytest.warns(FutureWarning):
-        layer.brush_shape = 'circle'
     layer.brush_size = 2  # equivalent to 18-connected 3D neighborhood
     layer.paint((1, 1, 1, 1), 1)
 
@@ -958,3 +964,335 @@ def test_fill_tensorstore():
         layer.fill((1, 4, 6, 7), 4)
         modified_labels = np.where(labels == 2, 4, labels)
         np.testing.assert_array_equal(modified_labels, np.asarray(data))
+
+
+@pytest.mark.parametrize(
+    'scale', list(itertools.product([-2, 2], [-0.5, 0.5], [-0.5, 0.5]))
+)
+def test_paint_3d_negative_scale(scale):
+    labels = np.zeros((3, 5, 11, 11), dtype=int)
+    labels_layer = Labels(
+        labels, scale=(1,) + scale, translate=(-200, 100, 100)
+    )
+    labels_layer.n_edit_dimensions = 3
+    labels_layer.brush_size = 8
+    labels_layer.paint((1, 2, 5, 5), 1)
+    np.testing.assert_array_equal(
+        np.sum(labels_layer.data, axis=(1, 2, 3)), [0, 95, 0]
+    )
+
+
+def test_rendering_init():
+    shape = (6, 10, 15)
+    np.random.seed(0)
+    data = np.random.randint(20, size=shape)
+    layer = Labels(data, rendering='iso_categorical')
+
+    assert layer.rendering == Rendering.ISO_CATEGORICAL.value
+
+
+def test_3d_video_and_3d_scale_translate_then_scale_translate_padded():
+    # See the GitHub issue for more details:
+    # https://github.com/napari/napari/issues/2967
+    data = np.zeros((3, 5, 11, 11), dtype=int)
+    labels = Labels(data, scale=(2, 1, 1), translate=(5, 5, 5))
+
+    np.testing.assert_array_equal(labels.scale, (1, 2, 1, 1))
+    np.testing.assert_array_equal(labels.translate, (0, 5, 5, 5))
+
+
+@dataclass
+class MouseEvent:
+    # mock mouse event class
+    pos: List[int]
+    position: List[int]
+    dims_point: List[int]
+    dims_displayed: List[int]
+    view_direction: List[int]
+
+
+def test_get_value_ray_3d():
+    """Test using _get_value_ray to interrogate labels in 3D"""
+    # make a mock mouse event
+    mouse_event = MouseEvent(
+        pos=[25, 25],
+        position=[10, 5, 5],
+        dims_point=[1, 0, 0, 0],
+        dims_displayed=[1, 2, 3],
+        view_direction=[1, 0, 0],
+    )
+    data = np.zeros((5, 20, 20, 20), dtype=int)
+    data[1, 0:10, 0:10, 0:10] = 1
+    labels = Labels(data, scale=(1, 2, 1, 1), translate=(5, 5, 5))
+
+    # set the dims to the slice with labels
+    labels._slice_dims([1, 0, 0, 0], ndisplay=3)
+
+    value = labels._get_value_ray(
+        start_point=np.array([1, 0, 5, 5]),
+        end_point=np.array([1, 20, 5, 5]),
+        dims_displayed=mouse_event.dims_displayed,
+    )
+    assert value == 1
+
+    # check with a ray that only goes through background
+    value = labels._get_value_ray(
+        start_point=np.array([1, 0, 15, 15]),
+        end_point=np.array([1, 20, 15, 15]),
+        dims_displayed=mouse_event.dims_displayed,
+    )
+    assert value is None
+
+    # set the dims to a slice without labels
+    labels._slice_dims([0, 0, 0, 0], ndisplay=3)
+
+    value = labels._get_value_ray(
+        start_point=np.array([0, 0, 5, 5]),
+        end_point=np.array([0, 20, 5, 5]),
+        dims_displayed=mouse_event.dims_displayed,
+    )
+    assert value is None
+
+
+def test_get_value_ray_3d_rolled():
+    """Test using _get_value_ray to interrogate labels in 3D
+    with the dimensions rolled.
+    """
+    # make a mock mouse event
+    mouse_event = MouseEvent(
+        pos=[25, 25],
+        position=[10, 5, 5, 1],
+        dims_point=[0, 0, 0, 1],
+        dims_displayed=[0, 1, 2],
+        view_direction=[1, 0, 0, 0],
+    )
+    data = np.zeros((20, 20, 20, 5), dtype=int)
+    data[0:10, 0:10, 0:10, 1] = 1
+    labels = Labels(data, scale=(1, 2, 1, 1), translate=(5, 5, 5, 0))
+
+    # set the dims to the slice with labels
+    labels._slice_dims((0, 0, 0, 1), ndisplay=3, order=(3, 0, 1, 2))
+    labels.set_view_slice()
+
+    value = labels._get_value_ray(
+        start_point=np.array([0, 5, 5, 1]),
+        end_point=np.array([20, 5, 5, 1]),
+        dims_displayed=mouse_event.dims_displayed,
+    )
+    assert value == 1
+
+
+def test_get_value_ray_3d_transposed():
+    """Test using _get_value_ray to interrogate labels in 3D
+    with the dimensions trasposed.
+    """
+    # make a mock mouse event
+    mouse_event = MouseEvent(
+        pos=[25, 25],
+        position=[10, 5, 5, 1],
+        dims_point=[0, 0, 0, 1],
+        dims_displayed=[1, 3, 2],
+        view_direction=[1, 0, 0, 0],
+    )
+    data = np.zeros((5, 20, 20, 20), dtype=int)
+    data[1, 0:10, 0:10, 0:10] = 1
+    labels = Labels(data, scale=(1, 2, 1, 1), translate=(0, 5, 5, 5))
+
+    # set the dims to the slice with labels
+    labels._slice_dims((1, 0, 0, 0), ndisplay=3, order=(0, 1, 3, 2))
+    labels.set_view_slice()
+
+    value = labels._get_value_ray(
+        start_point=np.array([1, 0, 5, 5]),
+        end_point=np.array([1, 20, 5, 5]),
+        dims_displayed=mouse_event.dims_displayed,
+    )
+    assert value == 1
+
+
+def test_get_value_ray_2d():
+    """_get_value_ray currently only returns None in 2D
+    (i.e., it shouldn't be used for 2D).
+    """
+    # make a mock mouse event
+    mouse_event = MouseEvent(
+        pos=[25, 25],
+        position=[5, 5],
+        dims_point=[1, 10, 0, 0],
+        dims_displayed=[2, 3],
+        view_direction=[1, 0, 0],
+    )
+    data = np.zeros((5, 20, 20, 20), dtype=int)
+    data[1, 0:10, 0:10, 0:10] = 1
+    labels = Labels(data, scale=(1, 2, 1, 1), translate=(5, 5, 5))
+
+    # set the dims to the slice with labels, but 2D
+    labels._slice_dims([1, 10, 0, 0], ndisplay=2)
+
+    value = labels._get_value_ray(
+        start_point=np.empty([]),
+        end_point=np.empty([]),
+        dims_displayed=mouse_event.dims_displayed,
+    )
+    assert value is None
+
+
+def test_cursor_ray_3d():
+    # make a mock mouse event
+    mouse_event_1 = MouseEvent(
+        pos=[25, 25],
+        position=[1, 10, 27, 10],
+        dims_point=[1, 0, 0, 0],
+        dims_displayed=[1, 2, 3],
+        view_direction=[0, 1, 0, 0],
+    )
+    data = np.zeros((5, 20, 20, 20), dtype=int)
+    data[1, 0:10, 0:10, 0:10] = 1
+    labels = Labels(data, scale=(1, 1, 2, 1), translate=(5, 5, 5))
+
+    # set the slice to one with data and the view to 3D
+    labels._slice_dims([1, 0, 0, 0], ndisplay=3)
+
+    # axis 0 : [0, 20], bounding box extents along view axis, [1, 0, 0]
+    # click is transformed: (value - translation) / scale
+    # axis 1: click at 27 in world coords -> (27 - 5) / 2 = 11
+    # axis 2: click at 10 in world coords -> (10 - 5) / 1 = 5
+    start_point, end_point = labels.get_ray_intersections(
+        mouse_event_1.position,
+        mouse_event_1.view_direction,
+        mouse_event_1.dims_displayed,
+    )
+    np.testing.assert_allclose(start_point, [1, 0, 11, 5])
+    np.testing.assert_allclose(end_point, [1, 19, 11, 5])
+
+    # click in the background
+    mouse_event_2 = MouseEvent(
+        pos=[25, 25],
+        position=[1, 10, 65, 10],
+        dims_point=[1, 0, 0, 0],
+        dims_displayed=[1, 2, 3],
+        view_direction=[0, 1, 0, 0],
+    )
+    start_point, end_point = labels.get_ray_intersections(
+        mouse_event_2.position,
+        mouse_event_2.view_direction,
+        mouse_event_2.dims_displayed,
+    )
+    assert start_point is None
+    assert end_point is None
+
+    # click in a slice with no labels
+    mouse_event_3 = MouseEvent(
+        pos=[25, 25],
+        position=[0, 10, 27, 10],
+        dims_point=[0, 0, 0, 0],
+        dims_displayed=[1, 2, 3],
+        view_direction=[0, 1, 0, 0],
+    )
+    labels._slice_dims([0, 0, 0, 0], ndisplay=3)
+    start_point, end_point = labels.get_ray_intersections(
+        mouse_event_3.position,
+        mouse_event_3.view_direction,
+        mouse_event_3.dims_displayed,
+    )
+    np.testing.assert_allclose(start_point, [0, 0, 11, 5])
+    np.testing.assert_allclose(end_point, [0, 19, 11, 5])
+
+
+def test_cursor_ray_3d_rolled():
+    """Test that the cursor works when the displayed
+    viewer axes have been rolled
+    """
+    # make a mock mouse event
+    mouse_event_1 = MouseEvent(
+        pos=[25, 25],
+        position=[10, 27, 10, 1],
+        dims_point=[0, 0, 0, 1],
+        dims_displayed=[0, 1, 2],
+        view_direction=[1, 0, 0, 0],
+    )
+    data = np.zeros((20, 20, 20, 5), dtype=int)
+    data[0:10, 0:10, 0:10, 1] = 1
+    labels = Labels(data, scale=(1, 2, 1, 1), translate=(5, 5, 5, 0))
+
+    # set the slice to one with data and the view to 3D
+    labels._slice_dims([0, 0, 0, 1], ndisplay=3)
+
+    start_point, end_point = labels.get_ray_intersections(
+        mouse_event_1.position,
+        mouse_event_1.view_direction,
+        mouse_event_1.dims_displayed,
+    )
+    np.testing.assert_allclose(start_point, [0, 11, 5, 1])
+    np.testing.assert_allclose(end_point, [19, 11, 5, 1])
+
+
+def test_cursor_ray_3d_transposed():
+    """Test that the cursor works when the displayed
+    viewer axes have been transposed
+    """
+    # make a mock mouse event
+    mouse_event_1 = MouseEvent(
+        pos=[25, 25],
+        position=[10, 27, 10, 1],
+        dims_point=[0, 0, 0, 1],
+        dims_displayed=[0, 2, 1],
+        view_direction=[1, 0, 0, 0],
+    )
+    data = np.zeros((20, 20, 20, 5), dtype=int)
+    data[0:10, 0:10, 0:10, 1] = 1
+    labels = Labels(data, scale=(1, 2, 1, 1), translate=(5, 5, 5, 0))
+
+    # set the slice to one with data and the view to 3D
+    labels._slice_dims([0, 0, 0, 1], ndisplay=3)
+
+    start_point, end_point = labels.get_ray_intersections(
+        mouse_event_1.position,
+        mouse_event_1.view_direction,
+        mouse_event_1.dims_displayed,
+    )
+    np.testing.assert_allclose(start_point, [0, 11, 5, 1])
+    np.testing.assert_allclose(end_point, [19, 11, 5, 1])
+
+
+def test_labels_state_update():
+    """Test that a labels layer can be updated from the output of its
+    _get_state() method
+    """
+    data = np.random.randint(20, size=(10, 15))
+    layer = Labels(data)
+    state = layer._get_state()
+    for k, v in state.items():
+        setattr(layer, k, v)
+
+
+def test_is_default_color():
+    """Test labels layer default color for None and background
+
+    Previously, setting color to just default values would
+    change color mode to DIRECT and display a black layer.
+    This test ensures `is_default_color` is
+    correctly checking against layer defaults, and `color_mode`
+    is only changed when appropriate.
+
+    See
+        - https://github.com/napari/napari/issues/2479
+        - https://github.com/napari/napari/issues/2953
+    """
+    data = np.random.randint(20, size=(10, 15))
+    layer = Labels(data)
+
+    # layer gets instantiated with defaults
+    current_color = layer.color
+    assert layer._is_default_colors(current_color)
+
+    # setting color to default colors doesn't update color mode
+    layer.color = current_color
+    assert layer.color_mode == 'auto'
+
+    # new colors are not default
+    new_color = {0: 'white', 1: 'red', 3: 'green'}
+    assert not layer._is_default_colors(new_color)
+    # setting the color with non-default colors updates color mode
+    layer.color = new_color
+    assert layer.color_mode == 'direct'
