@@ -7,6 +7,7 @@ from napari.layers.shapes import _shapes_key_bindings as key_bindings
 def test_lock_aspect_ratio():
     # Test a single four corner rectangle
     layer = Shapes(20 * np.random.random((1, 4, 2)))
+    layer._moving_coordinates = (0, 0, 0)
     layer._is_moving = True
     # need to go through the generator
     _ = list(key_bindings.hold_to_lock_aspect_ratio(layer))
@@ -17,6 +18,7 @@ def test_lock_aspect_ratio_selected_box():
     layer = Shapes(20 * np.random.random((1, 4, 2)))
     # select a shape
     layer._selected_box = layer.interaction_box(0)
+    layer._moving_coordinates = (0, 0, 0)
     layer._is_moving = True
     # need to go through the generator
     _ = list(key_bindings.hold_to_lock_aspect_ratio(layer))
@@ -27,6 +29,7 @@ def test_lock_aspect_ratio_selected_box_zeros():
     layer = Shapes(20 * np.zeros((1, 4, 2)))
     # select a shape
     layer._selected_box = layer.interaction_box(0)
+    layer._moving_coordinates = (0, 0, 0)
     layer._is_moving = True
     # need to go through the generator
     _ = list(key_bindings.hold_to_lock_aspect_ratio(layer))
@@ -58,7 +61,7 @@ def test_activate_modes():
     assert layer.mode == 'direct'
     key_bindings.activate_select_mode(layer)
     assert layer.mode == 'select'
-    key_bindings.activate_pan_zoom_mode(layer)
+    key_bindings.activate_shape_pan_zoom_mode(layer)
     assert layer.mode == 'pan_zoom'
     key_bindings.activate_vertex_insert_mode(layer)
     assert layer.mode == 'vertex_insert'
@@ -75,11 +78,11 @@ def test_copy_paste():
     assert layer._clipboard == {}
     layer.selected_data = {0, 1}
 
-    key_bindings.copy(layer)
+    key_bindings.copy_selected_shapes(layer)
     assert len(layer.data) == 3
     assert len(layer._clipboard) == 6
 
-    key_bindings.paste(layer)
+    key_bindings.paste_shape(layer)
     assert len(layer.data) == 5
     assert len(layer._clipboard) == 6
 
@@ -92,7 +95,7 @@ def test_select_all():
     assert len(layer.data) == 3
     assert len(layer.selected_data) == 0
 
-    key_bindings.select_all(layer)
+    key_bindings.select_all_shapes(layer)
     assert len(layer.selected_data) == 3
 
 
@@ -104,11 +107,11 @@ def test_delete():
     assert len(layer.data) == 3
     layer.selected_data = {0, 1}
 
-    key_bindings.delete_selected(layer)
+    key_bindings.delete_selected_shapes(layer)
     assert len(layer.data) == 1
 
 
 def test_finish():
     # Test on three four corner rectangle
     layer = Shapes(20 * np.random.random((3, 4, 2)))
-    key_bindings.finish_drawing(layer)
+    key_bindings.finish_drawing_shape(layer)
