@@ -2,12 +2,13 @@ import itertools
 
 import numpy as np
 import pytest
-import tensorstore as ts
-import torch
 import zarr
 from dask import array as da
 
-from napari.utils._dtype import normalize_dtype
+from napari.utils._dtype import get_dtype_limits, normalize_dtype
+
+ts = pytest.importorskip('tensorstore')
+torch = pytest.importorskip('torch')
 
 bit_depths = [str(2 ** i) for i in range(3, 7)]
 uints = ['uint' + b for b in bit_depths]
@@ -57,3 +58,24 @@ def test_pure_python_types(dtype_str):
 
 # note: we don't write specific tests for zarr and dask because they use numpy
 # dtypes directly.
+
+
+@pytest.mark.parametrize(
+    "dtype",
+    [
+        int,
+        'uint8',
+        np.uint8,
+        'int8',
+        'uint16',
+        'int16',
+        'uint32',
+        'int32',
+        float,
+        'float32',
+        'float64',
+    ],
+)
+def test_dtype_lims(dtype):
+    lims = get_dtype_limits(dtype)
+    assert isinstance(lims, tuple) and len(lims) == 2
